==> mr7.3.tar.gz: kamailio/src/modules/geoip2/geoip2_mod.c <==
/**
 *
 * This file is part of Kamailio, a free SIP server.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "../../core/sr_module.h"
#include "../../core/dprint.h"
#include "../../core/ut.h"
#include "../../core/pvar.h"
#include "../../core/kemi.h"
#include "../../core/mod_fix.h"

#include "geoip2_pv.h"

MODULE_VERSION

static char *geoip2_path = NULL;

static int  mod_init(void);
static void mod_destroy(void);

static int w_geoip2_match(struct sip_msg* msg, char* str1, char* str2);
static int geoip2_match(sip_msg_t *msg, str *tomatch, str *pvclass);

static pv_export_t mod_pvs[] = {
	{ {"gip2", sizeof("gip2")-1}, PVT_OTHER, pv_get_geoip2, 0,
		pv_parse_geoip2_name, 0, 0, 0 },
	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
};


static cmd_export_t cmds[]={
	{"geoip2_match", (cmd_function)w_geoip2_match, 2, fixup_spve_spve,
		0, ANY_ROUTE},
	{0, 0, 0, 0, 0, 0}
};

static param_export_t params[]={
	{"path",     PARAM_STRING, &geoip2_path},
	{0, 0, 0}
};

struct module_exports exports = {
	"geoip2",
	DEFAULT_DLFLAGS, /* dlopen flags */
	cmds,
	params,
	0,
	0,              /* exported MI functions */
	mod_pvs,        /* exported pseudo-variables */
	0,              /* extra processes */
	mod_init,       /* module initialization function */
	0,              /* response function */
	mod_destroy,    /* destroy function */
	0               /* per child init function */
};


/**
 * init module function
 */
static int mod_init(void)
{

	if(geoip2_path==NULL || strlen(geoip2_path)==0)
	{
		LM_ERR("path to GeoIP database file not set\n");
		return -1;
	}

	if(geoip2_init_pv(geoip2_path)!=0)
	{
		LM_ERR("cannot init for database file at: %s\n", geoip2_path);
		return -1;
	}
	return 0;
}

/**
 * destroy module function
 */
static void mod_destroy(void)
{
	geoip2_destroy_pv();
}

static int geoip2_match(sip_msg_t *msg, str *tomatch, str *pvclass)
{
	geoip2_pv_reset(pvclass);

	return geoip2_update_pv(tomatch, pvclass);
}

static int w_geoip2_match(sip_msg_t* msg, char* target, char* pvname)
{
	str tomatch = STR_NULL;
	str pvclass = STR_NULL;

	if(msg==NULL) {
		LM_ERR("received null msg\n");
		return -1;
	}

	if(fixup_get_svalue(msg, (gparam_t*)target, &tomatch)<0) {
		LM_ERR("cannot get the address\n");
		return -1;
	}
	if(fixup_get_svalue(msg, (gparam_t*)pvname, &pvclass)<0) {
		LM_ERR("cannot get the pv class\n");
		return -1;
	}

	return geoip2_match(msg, &tomatch, &pvclass);
}

/**
 *
 */
static sr_kemi_t sr_kemi_geoip2_exports[] = {
    { str_init("geoip2"), str_init("match"),
        SR_KEMIP_INT, geoip2_match,
        { SR_KEMIP_STR, SR_KEMIP_STR, SR_KEMIP_NONE,
            SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
    },

    { {0, 0}, {0, 0}, 0, NULL, { 0, 0, 0, 0, 0, 0 } }
};

int mod_register(char *path, int *dlflags, void *p1, void *p2) {
    sr_kemi_modules_add(sr_kemi_geoip2_exports);
    return 0;
}

==> mr11.2.1.tar.gz: kamailio/src/modules/geoip2/geoip2_mod.c <==
/**
 *
 * This file is part of Kamailio, a free SIP server.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "../../core/sr_module.h"
#include "../../core/dprint.h"
#include "../../core/ut.h"
#include "../../core/pvar.h"
#include "../../core/kemi.h"
#include "../../core/mod_fix.h"
#include "../../core/rpc_lookup.h"

#include "geoip2_pv.h"

MODULE_VERSION

static char *geoip2_path = NULL;

static int  mod_init(void);
static void mod_destroy(void);
static int geoip2_rpc_init(void);

static int w_geoip2_match(struct sip_msg* msg, char* str1, char* str2);
static int geoip2_match(sip_msg_t *msg, str *tomatch, str *pvclass);

static pv_export_t mod_pvs[] = {
	{ {"gip2", sizeof("gip2")-1}, PVT_OTHER, pv_get_geoip2, 0,
		pv_parse_geoip2_name, 0, 0, 0 },
	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
};


static cmd_export_t cmds[]={
	{"geoip2_match", (cmd_function)w_geoip2_match, 2, fixup_spve_spve,
		0, ANY_ROUTE},
	{0, 0, 0, 0, 0, 0}
};

static param_export_t params[]={
	{"path",     PARAM_STRING, &geoip2_path},
	{0, 0, 0}
};

struct module_exports exports = {
	"geoip2",			/* module name */
	DEFAULT_DLFLAGS,	/* dlopen flags */
	cmds,				/* exported functions */
	params,				/* exported parameters */
	0,					/* RPC method exports */
	mod_pvs,			/* exported pseudo-variables */
	0,					/* response handling function */
	mod_init,			/* module initialization function */
	0,					/* per-child init function */
	mod_destroy			/* module destroy function */
};


/**
 * init module function
 */
static int mod_init(void)
{
	LM_INFO("using GeoIP database path %s, library version %s\n", geoip2_path, MMDB_lib_version());

	if(geoip2_path==NULL || strlen(geoip2_path)==0)
	{
		LM_ERR("path to GeoIP database file not set\n");
		return -1;
	}

	if(geoip2_init_pv(geoip2_path)!=0)
	{
		LM_ERR("cannot init for database file at: %s\n", geoip2_path);
		return -1;
	}

	if (geoip2_rpc_init() < 0)
	{
		LM_ERR("error during RPC initialization\n");
		return -1;
	}

	return 0;
}

/**
 * destroy module function
 */
static void mod_destroy(void)
{
	geoip2_destroy_pv();
}

static int geoip2_match(sip_msg_t *msg, str *tomatch, str *pvclass)
{
	geoip2_pv_reset(pvclass);

	return geoip2_update_pv(tomatch, pvclass);
}

static int w_geoip2_match(sip_msg_t* msg, char* target, char* pvname)
{
	str tomatch = STR_NULL;
	str pvclass = STR_NULL;

	if(msg==NULL) {
		LM_ERR("received null msg\n");
		return -1;
	}

	if(fixup_get_svalue(msg, (gparam_t*)target, &tomatch)<0) {
		LM_ERR("cannot get the address\n");
		return -1;
	}
	if(fixup_get_svalue(msg, (gparam_t*)pvname, &pvclass)<0) {
		LM_ERR("cannot get the pv class\n");
		return -1;
	}

	return geoip2_match(msg, &tomatch, &pvclass);
}

static void geoip2_rpc_reload(rpc_t* rpc, void* ctx)
{
	if(geoip2_reload_pv(geoip2_path) != 0)
	{
		rpc->fault(ctx, 500, "Reload failed");
		return;
	}
}

static const char* geoip2_rpc_reload_doc[2] =
{
	"Reload GeoIP2 database file.",
	0
};

rpc_export_t ubl_rpc[] =
{
	{"geoip2.reload", geoip2_rpc_reload,
		geoip2_rpc_reload_doc, 0},
	{0, 0, 0, 0}
};

static int geoip2_rpc_init(void)
{
        if (rpc_register_array(ubl_rpc)!=0)
        {
                LM_ERR("failed to register RPC commands\n");
                return -1;
        }
        return 0;
}

/**
 *
 */
/* clang-format off */
static sr_kemi_t sr_kemi_geoip2_exports[] = {
    { str_init("geoip2"), str_init("match"),
        SR_KEMIP_INT, geoip2_match,
        { SR_KEMIP_STR, SR_KEMIP_STR, SR_KEMIP_NONE,
            SR_KEMIP_NONE, SR_KEMIP_NONE, SR_KEMIP_NONE }
    },

    { {0, 0}, {0, 0}, 0, NULL, { 0, 0, 0, 0, 0, 0 } }
};
/* clang-format on */

int mod_register(char *path, int *dlflags, void *p1, void *p2) {
    sr_kemi_modules_add(sr_kemi_geoip2_exports);
    return 0;
}
